﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading.Tasks;
using UnityEngine;

public class Type5mSelecterV3 : CelebrationSync
{
    public Type5mSelecterConfig config;
    [SerializeField] Type5mManagerV3[] manager = new Type5mManagerV3[2];
    [SerializeField] Animator[] animator = new Animator[3];
    [SerializeField] int lightColor = 1; //1:Red, 2:Green, 3:Blue, 4:Yellow, 5:Sky, 6:Purple, 7:Pink(255,128,255), 8:Orange(255,128,0), 9:Forest(128,255,0)
    private BGMPlayer bp;
    int nextPlay = 6;
    /// <summary>
    /// 左右どちらかのプレイヤーがプレイ中である
    /// </summary>
    bool isNowPlaying = false;

    /// <summary>
    /// 初回フラグ
    /// </summary>
    bool isFirst = true;

    // 以下はconfigに移動
    //[SerializeField, Range(0, 3), TooltipAttribute("BGMの切り替え 0,1: CATCHER 9、2: CATCHER 8、3: CATCHER 7")]
    //int soundType = 0; //0:U9_1，1:U9_2，2:U8，3:U7
    //[SerializeField] float audioPitch = 1.0f;

    //[SerializeField, TooltipAttribute("BGMを鳴らさない")] bool disableBgm = false;
    //[SerializeField] int waitBgmType = 0; // soundType=3のみ有効 0:通常 1:プレイ時と同じ 2:鳴らさない
    //                                      // TODO: soundType=1 諸事情でコイン投入で頭から再生にならない。おそらく後ろで待機BGMをVol=0で再生して管理したらいけるとは思う。

    void Start()
    {
        bp = transform.Find("BGM").GetComponent<BGMPlayer>();

        for (int i = 0; i < 3; i++)
        {
            animator[i].SetInteger("ColorNumber", lightColor);
            manager[0].animator[i] = animator[i];
            manager[1].animator[i] = animator[i];
        }

        if (config != null)
        {
            Setup(config);
        }
    }

    public void Setup(Type5mSelecterConfig config)
    {
        this.config = config;
        bp.SetAudioPitch(config.audioPitch);
        manager[0].soundType = config.soundType;
        manager[1].soundType = config.soundType;
        manager[0].sp.SetAudioPitch(config.audioPitch);
        manager[1].sp.SetAudioPitch(config.audioPitch);
    }

    async void Update()
    {
        // BGMを再生しない
        if (config.disableBgm)
            return;

        if (manager[0].craneStatus <= 0 && manager[1].craneStatus <= 0)
        {
            if (isNowPlaying || isFirst)
            {
                switch (config.soundType)
                {
                    case 0:
                        bp.Stop(1);
                        bp.Play(0);
                        break;
                    case 1:
                        bp.Stop(3);
                        bp.Play(2);
                        break;
                    case 2:
                        bp.Stop(5);
                        bp.Play(4);
                        break;
                    case 3:
                        switch (config.waitBgmType)
                        {
                            case 0:
                                bp.Stop(8);
                                if (!bp.audioSources[6].isPlaying && !bp.audioSources[7].isPlaying)
                                {
                                    bp.Play(nextPlay);
                                }
                                break;
                            case 1:
                                bp.Stop(6);
                                bp.Stop(7);
                                bp.Stop(8);
                                await Task.Delay(400);
                                bp.Play(8);
                                break;
                            case 2:
                                bp.Stop(6);
                                bp.Stop(7);
                                bp.Stop(8);
                                break;

                            default:
                                break;
                        }
                        break;
                }
                isNowPlaying = false;
                isFirst = false;
            }
        }
        else if (manager[0].craneStatus > 0 || manager[1].craneStatus > 0)
        {
            if (!isNowPlaying)
            {
                switch (config.soundType)
                {
                    case 0:
                        bp.Stop(0);
                        bp.Play(1);
                        break;
                    case 1:
                        bp.Stop(2);
                        bp.Play(3);
                        break;
                    case 2:
                        bp.Stop(4);
                        bp.Play(5);
                        break;
                    case 3:
                        bp.Stop(6);
                        bp.Stop(7);
                        await Task.Delay(400);
                        if (config.waitBgmType != 1)
                        {
                            // waitBgmType=1(外周BGMとプレイBGMが同じ)でない場合
                            bp.Play(8);
                        }
                        else
                        {
                            bp.Stop(8);
                            bp.Play(8);
                        }
                        // 次のに再生する待機中BGMを指定
                        if (nextPlay == 6) nextPlay = 7;
                        else if (nextPlay == 7) nextPlay = 6;
                        break;
                }
                isNowPlaying = true;
            }
        }
    }

    //public Type5mManager GetManager(int num)
    //{
    //    return manager[num - 1];
    //}

    public override void SetupNetWork(Type5NetworkV3 net) // 獲得連動用
    {
        manager[0].net = net;
        manager[1].net = net;
    }

    public override void Celebrate() // 獲得連動用
    {
        for (int i = 0; i < 3; i++) animator[i].SetTrigger("GetPrize");
    }
}
